@@ -36,6 +36,8 @@
 #include "rt_numa.h"
 
 #include "rt-utils.h"
+#include "rt-numa.h"
+#include "rt-error.h"
 
 #include <bionic.h>
 
@@ -62,12 +64,12 @@ static int clock_nanosleep(clockid_t clock_id, int flags, const struct timespec
 	if (clock_id == CLOCK_THREAD_CPUTIME_ID)
 		return -EINVAL;
 	if (clock_id == CLOCK_PROCESS_CPUTIME_ID)
-		clock_id = MAKE_PROCESS_CPUCLOCK (0, CPUCLOCK_SCHED);
+		clock_id = MAKE_PROCESS_CPUCLOCK(0, CPUCLOCK_SCHED);
 
 	return syscall(__NR_clock_nanosleep, clock_id, flags, req, rem);
 }
 
-int sched_setaffinity (__pid_t __pid, size_t __cpusetsize,
+int sched_setaffinity(__pid_t __pid, size_t __cpusetsize,
 		       __const cpu_set_t *__cpuset)
 {
 	return -EINVAL;
@@ -216,6 +218,7 @@ static struct timespec globalt;
 
 static char fifopath[MAX_PATH];
 static char histfile[MAX_PATH];
+static char jsonfile[MAX_PATH];
 
 static struct thread_param **parameters;
 static struct thread_stat **statistics;
@@ -283,16 +286,13 @@ enum {
 	ERROR_NOTFOUND	= -2,
 };
 
-static int trace_fd     = -1;
-static int tracemark_fd = -1;
-
 /*
  * Raise the soft priority limit up to prio, if that is less than or equal
  * to the hard limit
  * if a call fails, return the error
  * if successful return 0
  * if fails, return -1
-*/
+ */
 static int raise_soft_prio(int policy, const struct sched_param *param)
 {
 	int err;
@@ -352,7 +352,8 @@ try_again:
 		if (err == EPERM) {
 			int err1;
 			err1 = raise_soft_prio(policy, param);
-			if (!err1) goto try_again;
+			if (!err1)
+				goto try_again;
 		}
 	}
 
@@ -551,7 +552,7 @@ static void *timerthread(void *param)
 		fatal("timerthread%d: failed to set priority to %d\n",
 		      par->cpu, par->prio);
 
-	if(smi) {
+	if (smi) {
 		par->msr_fd = open_msr_file(par->cpu);
 		if (par->msr_fd < 0)
 			fatal("Could not open MSR interface, errno: %d\n",
@@ -569,7 +570,7 @@ static void *timerthread(void *param)
 			clock_gettime(par->clock, &globalt);
 			if (secaligned) {
 				/* Ensure that the thread start timestamp is not
-				   in the past */
+				 * in the past */
 				if (globalt.tv_nsec > 900000000)
 					globalt.tv_sec += 2;
 				else
@@ -614,6 +615,7 @@ static void *timerthread(void *param)
 		setitimer(ITIMER_REAL, &itimer, NULL);
 	}
 
+	rt_test_start();
 	stat->threadstarted++;
 
 	while (!shutdown) {
@@ -709,23 +711,22 @@ static void *timerthread(void *param)
 		}
 		stat->avg += (double) diff;
 
-		if (trigger && (diff > trigger)) {
+		if (trigger && (diff > trigger))
 			trigger_update(par, diff, calctime(now));
-		}
-
 
 		if (duration && (calcdiff(now, stop) >= 0))
 			shutdown++;
 
 		if (!stopped && tracelimit && (diff > tracelimit)) {
 			stopped++;
-			tracemark("hit latency threshold (%llu > %d)",
-				  (unsigned long long) diff, tracelimit);
 			shutdown++;
 			pthread_mutex_lock(&break_thread_id_lock);
-			if (break_thread_id == 0)
+			if (break_thread_id == 0) {
 				break_thread_id = stat->tid;
-			break_thread_value = diff;
+				tracemark("hit latency threshold (%llu > %d)",
+					  (unsigned long long) diff, tracelimit);
+				break_thread_value = diff;
+			}
 			pthread_mutex_unlock(&break_thread_id_lock);
 		}
 		stat->act = diff;
@@ -809,17 +810,12 @@ static void display_help(int error)
 	printf("cyclictest V %1.2f\n", VERSION);
 	printf("Usage:\n"
 	       "cyclictest <options>\n\n"
-#if LIBNUMA_API_VERSION >= 2
 	       "-a [CPUSET] --affinity     Run thread #N on processor #N, if possible, or if CPUSET\n"
 	       "                           given, pin threads to that set of processors in round-\n"
 	       "                           robin order.  E.g. -a 2 pins all threads to CPU 2,\n"
 	       "                           but -a 3-5,0 -t 5 will run the first and fifth\n"
 	       "                           threads on CPU (0),thread #2 on CPU 3, thread #3\n"
 	       "                           on CPU 4, and thread #5 on CPU 5.\n"
-#else
-	       "-a [NUM] --affinity        run thread #N on processor #N, if possible\n"
-	       "                           with NUM pin all threads to the processor NUM\n"
-#endif
 	       "-A USEC  --aligned=USEC    align thread wakeups to a specific offset\n"
 	       "-b USEC  --breaktrace=USEC send break trace command when latency > USEC\n"
 	       "-c CLOCK --clock=CLOCK     select clock\n"
@@ -831,15 +827,19 @@ static void display_help(int error)
 	       "	 --latency=PM_QOS  write PM_QOS to /dev/cpu_dma_latency\n"
 	       "-F       --fifo=<path>     create a named pipe at path and write stats to it\n"
 	       "-h       --histogram=US    dump a latency histogram to stdout after the run\n"
-	       "                           US is the max latency time to be be tracked in microseconds\n"
+	       "                           US is the max latency time to be tracked in microseconds\n"
 	       "			   This option runs all threads at the same priority.\n"
 	       "-H       --histofall=US    same as -h except with an additional summary column\n"
 	       "	 --histfile=<path> dump the latency histogram to <path> instead of stdout\n"
 	       "-i INTV  --interval=INTV   base interval of thread in us default=1000\n"
+	       "         --json=FILENAME   write final results into FILENAME, JSON formatted\n"
 	       "-l LOOPS --loops=LOOPS     number of loops: default=0(endless)\n"
 	       "	 --laptop	   Save battery when running cyclictest\n"
 	       "			   This will give you poorer realtime results\n"
 	       "			   but will not drain your battery so quickly\n"
+	       "         --mainaffinity=CPUSET\n"
+	       "			   Run the main thread on CPU #N. This only affects\n"
+	       "                           the main thread and not the measurement threads\n"
 	       "-m       --mlockall        lock current and future memory allocations\n"
 	       "-M       --refresh_on_max  delay updating the screen until a new max\n"
 	       "			   latency is hit. Useful for low bandwidth.\n"
@@ -894,13 +894,8 @@ static int quiet;
 static int interval = DEFAULT_INTERVAL;
 static int distance = -1;
 static struct bitmask *affinity_mask = NULL;
+static struct bitmask *main_affinity_mask = NULL;
 static int smp = 0;
-
-enum {
-	AFFINITY_UNSPECIFIED,
-	AFFINITY_SPECIFIED,
-	AFFINITY_USEALL
-};
 static int setaffinity = AFFINITY_UNSPECIFIED;
 
 static int clocksources[] = {
@@ -908,141 +903,6 @@ static int clocksources[] = {
 	CLOCK_REALTIME,
 };
 
-static unsigned int is_cpumask_zero(const struct bitmask *mask)
-{
-	return (rt_numa_bitmask_count(mask) == 0);
-}
-
-/* Get available cpus according to getaffinity or according to the
- * intersection of getaffinity and the user specified affinity
- * in the case of AFFINITY_SPECIFIED, the function has to be called
- * after the call to parse_cpumask made in process_options()
- */
-static int get_available_cpus(void)
-{
-	int num_cpus = 0;
-	int res;
-	pthread_t thread;
-	cpu_set_t cpuset;
-
-
-	if (affinity_mask != NULL) {
-		num_cpus = rt_numa_bitmask_count(affinity_mask);
-	} else {
-		CPU_ZERO(&cpuset);
-		thread = pthread_self();
-		res = pthread_getaffinity_np(thread, sizeof(cpu_set_t), &cpuset);
-		if (res != 0) {
-			fatal("pthread_getaffinity_np failed: %s\n", strerror(res));
-		}
-		num_cpus = CPU_COUNT(&cpuset);
-	}
-
-	return num_cpus;
-
-}
-
-/* cpu_for_thread AFFINITY_SPECIFIED */
-static int cpu_for_thread_sp(int thread_num, int max_cpus)
-{
-	unsigned int m, cpu, i, num_cpus;
-
-	num_cpus = rt_numa_bitmask_count(affinity_mask);
-
-	if (num_cpus == 0) {
-		fatal("No allowable cpus to run on\n");
-	}
-
-	m = thread_num % num_cpus;
-
-	/* there are num_cpus bits set, we want position of m'th one */
-	for (i = 0, cpu = 0; i < max_cpus; i++) {
-		if (rt_numa_bitmask_isbitset(affinity_mask, i)) {
-			if (cpu == m)
-				return i;
-			cpu++;
-		}
-	}
-	fprintf(stderr, "Bug in cpu mask handling code.\n");
-	return 0;
-}
-
-/* cpu_for_thread AFFINITY_USEALL */
-static int cpu_for_thread_ua(int thread_num, int max_cpus)
-{
-	int res, num_cpus, i, m, cpu;
-	pthread_t thread;
-	cpu_set_t cpuset;
-
-	thread = pthread_self();
-	CPU_ZERO(&cpuset);
-
-	res = pthread_getaffinity_np(thread, sizeof(cpu_set_t), &cpuset);
-	if (res != 0) {
-		fatal("pthread_getaffinity_np failed: %s\n", strerror(res));
-	}
-
-	num_cpus = CPU_COUNT(&cpuset);
-	m = thread_num % num_cpus;
-
-	for (i = 0, cpu = 0; i < max_cpus; i++) {
-		if (CPU_ISSET(i, &cpuset)) {
-			if (cpu == m)
-				return i;
-			cpu++;
-		}
-	}
-
-	fprintf(stderr, "Bug in cpu mask handling code.\n");
-	return 0;
-}
-
-
-/* After this function is called, affinity_mask is the intersection of the user
- * supplied affinity mask and the affinity mask from the run time environment */
-static void use_current_cpuset(const int max_cpus)
-{
-	int i;
-	pid_t pid;
-	struct bitmask *curmask;
-
-	pid = getpid();
-
-	curmask = numa_allocate_cpumask();
-	numa_sched_getaffinity(pid, curmask);
-
-	/* Clear bits that are not set in both the cpuset from the environment,
-	 * and in the user specified affinity for cyclictest */
-	for (i=0; i < max_cpus; i++) {
-		if ((!rt_numa_bitmask_isbitset(affinity_mask, i)) || (!rt_numa_bitmask_isbitset(curmask, i))) {
-			numa_bitmask_clearbit(affinity_mask, i);
-		}
-	}
-
-	numa_bitmask_free(curmask);
-}
-
-static void parse_cpumask(const char *option, const int max_cpus)
-{
-	affinity_mask = rt_numa_parse_cpustring(option, max_cpus);
-	if (affinity_mask) {
-		if (is_cpumask_zero(affinity_mask)) {
-			rt_bitmask_free(affinity_mask);
-			affinity_mask = NULL;
-		} else {
-			use_current_cpuset(max_cpus);
-		}
-
-	}
-	if (!affinity_mask)
-		display_help(1);
-
-	if (verbose) {
-		printf("%s: Using %u cpus.\n", __func__,
-			rt_numa_bitmask_count(affinity_mask));
-	}
-}
-
 static void handlepolicy(char *polname)
 {
 	if (strncasecmp(polname, "other", 5) == 0)
@@ -1088,8 +948,8 @@ enum option_values {
 	OPT_AFFINITY=1, OPT_BREAKTRACE, OPT_CLOCK,
 	OPT_DISTANCE, OPT_DURATION, OPT_LATENCY,
 	OPT_FIFO, OPT_HISTOGRAM, OPT_HISTOFALL, OPT_HISTFILE,
-	OPT_INTERVAL, OPT_LOOPS, OPT_MLOCKALL, OPT_REFRESH,
-	OPT_NANOSLEEP, OPT_NSECS, OPT_OSCOPE, OPT_PRIORITY,
+	OPT_INTERVAL, OPT_JSON, OPT_MAINAFFINITY, OPT_LOOPS, OPT_MLOCKALL,
+	OPT_REFRESH, OPT_NANOSLEEP, OPT_NSECS, OPT_OSCOPE, OPT_PRIORITY,
 	OPT_QUIET, OPT_PRIOSPREAD, OPT_RELATIVE, OPT_RESOLUTION,
 	OPT_SYSTEM, OPT_SMP, OPT_THREADS, OPT_TRIGGER,
 	OPT_TRIGGER_NODES, OPT_UNBUFFERED, OPT_NUMA, OPT_VERBOSE,
@@ -1098,22 +958,8 @@ enum option_values {
 	OPT_TRACEMARK, OPT_POSIX_TIMERS,
 };
 
-/* numa_available() must be called before any other calls to the numa library */
-static void numa_initialize(void)
-{
-	static int is_initialized;
-
-	if (is_initialized == 1)
-		return;
-
-	if (numa_available() != -1)
-		numa = 1;
-
-	is_initialized = 1;
-}
-
 /* Process commandline options */
-static void process_options (int argc, char *argv[], int max_cpus)
+static void process_options(int argc, char *argv[], int max_cpus)
 {
 	int error = 0;
 	int option_affinity = 0;
@@ -1137,8 +983,10 @@ static void process_options (int argc, char *argv[], int max_cpus)
 			{"histofall",        required_argument, NULL, OPT_HISTOFALL },
 			{"histfile",	     required_argument, NULL, OPT_HISTFILE },
 			{"interval",         required_argument, NULL, OPT_INTERVAL },
+			{"json",             required_argument, NULL, OPT_JSON },
 			{"laptop",	     no_argument,	NULL, OPT_LAPTOP },
 			{"loops",            required_argument, NULL, OPT_LOOPS },
+			{"mainaffinity",     required_argument, NULL, OPT_MAINAFFINITY},
 			{"mlockall",         no_argument,       NULL, OPT_MLOCKALL },
 			{"refresh_on_max",   no_argument,       NULL, OPT_REFRESH },
 			{"nsecs",            no_argument,       NULL, OPT_NSECS },
@@ -1175,23 +1023,32 @@ static void process_options (int argc, char *argv[], int max_cpus)
 			/* smp sets AFFINITY_USEALL in OPT_SMP */
 			if (smp)
 				break;
-			numa_initialize();
-			if (optarg != NULL) {
-				parse_cpumask(optarg, max_cpus);
+			numa = numa_initialize();
+			if (optarg) {
+				parse_cpumask(optarg, max_cpus, &affinity_mask);
 				setaffinity = AFFINITY_SPECIFIED;
-			} else if (optind<argc && (atoi(argv[optind]) || argv[optind][0] == '0' || argv[optind][0] == '!')) {
-				parse_cpumask(argv[optind], max_cpus);
+			} else if (optind < argc &&
+				   (atoi(argv[optind]) ||
+				    argv[optind][0] == '0' ||
+				    argv[optind][0] == '!')) {
+				parse_cpumask(argv[optind], max_cpus, &affinity_mask);
 				setaffinity = AFFINITY_SPECIFIED;
 			} else {
 				setaffinity = AFFINITY_USEALL;
 			}
+
+			if (setaffinity == AFFINITY_SPECIFIED && !affinity_mask)
+				display_help(1);
+			if (verbose)
+				printf("Using %u cpus.\n",
+					numa_bitmask_weight(affinity_mask));
 			break;
 		case 'A':
 		case OPT_ALIGNED:
-			aligned=1;
+			aligned = 1;
 			if (optarg != NULL)
 				offset = atoi(optarg) * 1000;
-			else if (optind<argc && atoi(argv[optind]))
+			else if (optind < argc && atoi(argv[optind]))
 				offset = atoi(argv[optind]) * 1000;
 			else
 				offset = 0;
@@ -1228,9 +1085,22 @@ static void process_options (int argc, char *argv[], int max_cpus)
 		case 'i':
 		case OPT_INTERVAL:
 			interval = atoi(optarg); break;
+		case OPT_JSON:
+			strncpy(jsonfile, optarg, strnlen(optarg, MAX_PATH-1));
+			break;
 		case 'l':
 		case OPT_LOOPS:
 			max_cycles = atoi(optarg); break;
+		case OPT_MAINAFFINITY:
+			if (optarg) {
+				parse_cpumask(optarg, max_cpus, &main_affinity_mask);
+			} else if (optind < argc &&
+			           (atoi(argv[optind]) ||
+			            argv[optind][0] == '0' ||
+			            argv[optind][0] == '!')) {
+				parse_cpumask(argv[optind], max_cpus, &main_affinity_mask);
+			}
+			break;
 		case 'm':
 		case OPT_MLOCKALL:
 			lockall = 1; break;
@@ -1286,7 +1156,7 @@ static void process_options (int argc, char *argv[], int max_cpus)
 			}
 			if (optarg != NULL)
 				num_threads = atoi(optarg);
-			else if (optind<argc && atoi(argv[optind]))
+			else if (optind < argc && atoi(argv[optind]))
 				num_threads = atoi(argv[optind]);
 			else
 				num_threads = -1; /* update after parsing */
@@ -1347,14 +1217,9 @@ static void process_options (int argc, char *argv[], int max_cpus)
 
 	/* if smp wasn't requested, test for numa automatically */
 	if (!smp) {
-#ifdef NUMA
-		numa_initialize();
+		numa = numa_initialize();
 		if (setaffinity == AFFINITY_UNSPECIFIED)
 			setaffinity = AFFINITY_USEALL;
-#else
-		warn("cyclictest was not built with the numa option\n");
-		numa = 0;
-#endif
 	}
 
 	if (option_affinity) {
@@ -1397,7 +1262,7 @@ static void process_options (int argc, char *argv[], int max_cpus)
 		error = 1;
 
 	if (num_threads == -1)
-		num_threads = get_available_cpus();
+		num_threads = get_available_cpus(affinity_mask);
 
 	if (priospread && priority == 0) {
 		fprintf(stderr, "defaulting realtime priority to %d\n",
@@ -1606,7 +1471,7 @@ static void print_stat(FILE *fp, struct thread_param *par, int index, int verbos
 				(long)(stat->avg/stat->cycles) : 0, stat->max);
 
 			if (smi)
-				fprintf(fp," SMI:%8ld", stat->smi_count);
+				fprintf(fp, " SMI:%8ld", stat->smi_count);
 
 			fprintf(fp, "\n");
 		}
@@ -1662,7 +1527,7 @@ static void rstat_print_stat(struct thread_param *par, int index, int verbose, i
 				(long)(stat->avg/stat->cycles) : 0, stat->max);
 
 			if (smi)
-				dprintf(fd," SMI:%8ld", stat->smi_count);
+				dprintf(fd, " SMI:%8ld", stat->smi_count);
 
 			dprintf(fd, "\n");
 		}
@@ -1761,7 +1626,8 @@ static void trigger_print()
 	struct thread_trigger *trig = head;
 	char *fmt = "T:%2d Spike:%8ld: TS: %12ld\n";
 
-	if (current == head) return;
+	if (current == head)
+		return;
 	printf("\n");
 	while (trig->next != current) {
 		fprintf(stdout, fmt,  trig->tnum, trig->diff, trig->ts);
@@ -1804,9 +1670,8 @@ static int rstat_shm_open(void)
 
 	errno = 9;
 	fd = shm_open(shm_name, O_RDWR|O_CREAT|O_EXCL, S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP|S_IROTH|S_IWOTH);
-	if (fd == -1) {
+	if (fd == -1)
 		fprintf(stderr, "ERROR: shm_open %s\n", strerror(errno));
-	}
 
 	rstat_fd = fd;
 
@@ -1819,9 +1684,8 @@ static int rstat_ftruncate(int fd, off_t len)
 
 	errno = 0;
 	err = ftruncate(fd, len);
-	if (err) {
+	if (err)
 		fprintf(stderr, "ftruncate error %s\n", strerror(errno));
-	}
 
 	return err;
 }
@@ -1833,9 +1697,8 @@ static void *rstat_mmap(int fd)
 	errno = 0;
 	mptr = mmap(0, _SC_PAGE_SIZE, PROT_READ|PROT_WRITE, MAP_SHARED, fd, 0);
 
-	if (mptr == (void*)-1) {
+	if (mptr == (void*)-1)
 		fprintf(stderr, "ERROR: mmap, %s\n", strerror(errno));
-	}
 
 	return mptr;
 }
@@ -1846,9 +1709,8 @@ static int rstat_mlock(void *mptr)
 
 	errno = 0;
 	err = mlock(mptr, _SC_PAGE_SIZE);
-	if (err == -1) {
+	if (err == -1)
 		fprintf(stderr, "ERROR, mlock %s\n", strerror(errno));
-	}
 
 	return err;
 }
@@ -1886,6 +1748,51 @@ rstat_err:
 	return;
 }
 
+static void write_stats(FILE *f, void *data)
+{
+	struct thread_param **par = parameters;
+	unsigned int i, j, comma;
+	struct thread_stat *s;
+
+	fprintf(f, "  \"num_threads\": %d,\n", num_threads);
+	fprintf(f, "  \"resolution_in_ns\": %u,\n", use_nsecs);
+	fprintf(f, "  \"thread\": {\n");
+	for (i = 0; i < num_threads; i++) {
+		fprintf(f, "    \"%u\": {\n", i);
+
+		fprintf(f, "      \"histogram\": {");
+		s = par[i]->stats;
+		for (j = 0, comma = 0; j < histogram; j++) {
+			if (s->hist_array[j] == 0)
+				continue;
+			fprintf(f, "%s", comma ? ",\n" : "\n");
+			fprintf(f, "        \"%u\": %ld", j, s->hist_array[j]);
+			comma = 1;
+		}
+		if (comma)
+			fprintf(f, "\n");
+		fprintf(f, "      },\n");
+		fprintf(f, "      \"cycles\": %ld,\n", s->cycles);
+		fprintf(f, "      \"min\": %ld,\n", s->min);
+		fprintf(f, "      \"max\": %ld,\n", s->max);
+		fprintf(f, "      \"avg\": %.2f,\n", s->avg/s->cycles);
+		fprintf(f, "      \"cpu\": %d,\n", par[i]->cpu);
+		fprintf(f, "      \"node\": %d\n", par[i]->node);
+		fprintf(f, "    }%s\n", i == num_threads - 1 ? "" : ",");
+	}
+	fprintf(f, "  }\n");
+}
+
+static void set_main_thread_affinity(struct bitmask *cpumask)
+{
+	int res;
+
+	errno = 0;
+	res = numa_sched_setaffinity(getpid(), cpumask);
+	if (res != 0)
+		warn("Couldn't setaffinity in main thread: %s\n",
+		     strerror(errno));
+}
 
 int main(int argc, char **argv)
 {
@@ -1893,27 +1800,30 @@ int main(int argc, char **argv)
 	int signum = SIGALRM;
 	int mode;
 	int cpu;
-	int max_cpus = sysconf(_SC_NPROCESSORS_ONLN);
+	int max_cpus = sysconf(_SC_NPROCESSORS_CONF);
+	int online_cpus = sysconf(_SC_NPROCESSORS_ONLN);
 	int i, ret = -1;
 	int status;
 
+	rt_init(argc, argv);
 	process_options(argc, argv, max_cpus);
 
 	if (check_privs())
 		exit(EXIT_FAILURE);
 
-	if (verbose)
+	if (verbose) {
 		printf("Max CPUs = %d\n", max_cpus);
+		printf("Online CPUs = %d\n", online_cpus);
+	}
 
 	/* Restrict the main pid to the affinity specified by the user */
-	if (affinity_mask != NULL) {
-		int res;
-
-		errno = 0;
-		res = numa_sched_setaffinity(getpid(), affinity_mask);
-		if (res != 0) {
-			warn("Couldn't setaffinity in main thread: %s\n", strerror(errno));
-		}
+	if (main_affinity_mask != NULL) {
+		set_main_thread_affinity(main_affinity_mask);
+	} else if (affinity_mask != NULL) {
+		set_main_thread_affinity(affinity_mask);
+		if (verbose)
+			printf("Using %u cpus.\n",
+				numa_bitmask_weight(affinity_mask));
 	}
 
 	if (trigger) {
@@ -1955,11 +1865,10 @@ int main(int argc, char **argv)
 
 		clock = clocksources[clocksel];
 
-		if (clock_getres(clock, &res)) {
+		if (clock_getres(clock, &res))
 			warn("clock_getres failed");
-		} else {
+		else
 			reported_resolution = (NSEC_PER_SEC * res.tv_sec) + res.tv_nsec;
-		}
 
 
 		/*
@@ -1970,9 +1879,8 @@ int main(int argc, char **argv)
 		 */
 		times = 1000;
 		clock_gettime(clock, &prev);
-		for (k=0; k < times; k++) {
+		for (k=0; k < times; k++)
 			clock_gettime(clock, &now);
-		}
 
 		diff = calcdiff_ns(now, prev);
 		if (diff == 0) {
@@ -1995,9 +1903,8 @@ int main(int argc, char **argv)
 
 		time = calloc(times, sizeof(*time));
 
-		for (k=0; k < times; k++) {
+		for (k=0; k < times; k++)
 			clock_gettime(clock, &time[k]);
-		}
 
 		if (ct_debug) {
 			info("For %d consecutive calls to clock_gettime():\n", times);
@@ -2010,9 +1917,8 @@ int main(int argc, char **argv)
 			diff = calcdiff_ns(time[k], prev);
 			prev = time[k];
 
-			if (diff && (diff < min_non_zero_diff)) {
+			if (diff && (diff < min_non_zero_diff))
 				min_non_zero_diff = diff;
-			}
 
 			if (ct_debug)
 				info("%ld.%06ld  %5llu\n",
@@ -2043,7 +1949,7 @@ int main(int argc, char **argv)
 
 	sigemptyset(&sigset);
 	sigaddset(&sigset, signum);
-	sigprocmask (SIG_BLOCK, &sigset, NULL);
+	sigprocmask(SIG_BLOCK, &sigset, NULL);
 
 	signal(SIGINT, sighand);
 	signal(SIGTERM, sighand);
@@ -2073,7 +1979,7 @@ int main(int argc, char **argv)
 		switch (setaffinity) {
 		case AFFINITY_UNSPECIFIED: cpu = -1; break;
 		case AFFINITY_SPECIFIED:
-			cpu = cpu_for_thread_sp(i, max_cpus);
+			cpu = cpu_for_thread_sp(i, max_cpus, affinity_mask);
 			if (verbose)
 				printf("Thread %d using cpu %d.\n", i, cpu);
 			break;
@@ -2092,7 +1998,7 @@ int main(int argc, char **argv)
 			/* find the memory node associated with the cpu i */
 			node = rt_numa_numa_node_of_cpu(cpu);
 
-			/* get the stack size set for for this thread */
+			/* get the stack size set for this thread */
 			if (pthread_attr_getstack(&attr, &currstk, &stksize))
 				fatal("failed to get stack size for thread %d\n", i);
 
@@ -2223,7 +2129,7 @@ int main(int argc, char **argv)
 		for (i = 0; i < num_threads; i++) {
 
 			print_stat(stdout, parameters[i], i, verbose, quiet);
-			if(max_cycles && statistics[i]->cycles >= max_cycles)
+			if (max_cycles && statistics[i]->cycles >= max_cycles)
 				allstopped++;
 		}
 
@@ -2250,6 +2156,9 @@ int main(int argc, char **argv)
 	if (!verbose && !quiet && refresh_on_max)
 		printf("\033[%dB", num_threads + 2);
 
+	if (strlen(jsonfile) != 0)
+		rt_write_json(jsonfile, ret, write_stats, NULL);
+
 	if (quiet)
 		quiet = 2;
 	for (i = 0; i < num_threads; i++) {
@@ -2298,10 +2207,7 @@ int main(int argc, char **argv)
 	}
  out:
 	/* close any tracer file descriptors */
-	if (tracemark_fd >= 0)
-		close(tracemark_fd);
-	if (trace_fd >= 0)
-		close(trace_fd);
+	disable_trace_mark();
 
 	/* unlock everything */
 	if (lockall)
